@@ -28,7 +28,6 @@
 
 #include "taglib_export.h"
 #include "audioproperties.h"
-
 #include "mpegheader.h"
 
 namespace TagLib {
@@ -57,49 +56,32 @@ namespace TagLib {
       /*!
        * Destroys this MPEG Properties instance.
        */
-      virtual ~Properties();
+      ~Properties() override;
 
-      /*!
-       * Returns the length of the file in seconds.  The length is rounded down to
-       * the nearest whole second.
-       *
-       * \note This method is just an alias of lengthInSeconds().
-       *
-       * \deprecated Use lengthInSeconds().
-       */
-      TAGLIB_DEPRECATED virtual int length() const;
-
-      /*!
-       * Returns the length of the file in seconds.  The length is rounded down to
-       * the nearest whole second.
-       *
-       * \see lengthInMilliseconds()
-       */
-      // BIC: make virtual
-      int lengthInSeconds() const;
+      Properties(const Properties &) = delete;
+      Properties &operator=(const Properties &) = delete;
 
       /*!
        * Returns the length of the file in milliseconds.
        *
        * \see lengthInSeconds()
        */
-      // BIC: make virtual
-      int lengthInMilliseconds() const;
+      int lengthInMilliseconds() const override;
 
       /*!
        * Returns the average bit rate of the file in kb/s.
        */
-      virtual int bitrate() const;
+      int bitrate() const override;
 
       /*!
        * Returns the sample rate in Hz.
        */
-      virtual int sampleRate() const;
+      int sampleRate() const override;
 
       /*!
        * Returns the number of audio channels.
        */
-      virtual int channels() const;
+      int channels() const override;
 
       /*!
        * Returns a pointer to the Xing/VBRI header if one exists or null if no
@@ -118,7 +100,7 @@ namespace TagLib {
       int layer() const;
 
       /*!
-       * Returns true if the MPEG protection bit is enabled.
+       * Returns \c true if the MPEG protection bit is enabled.
        */
       bool protectionEnabled() const;
 
@@ -128,25 +110,33 @@ namespace TagLib {
       Header::ChannelMode channelMode() const;
 
       /*!
-       * Returns true if the copyrighted bit is set.
+       * Returns the MPEG-4 channel configuration.
+       */
+      Header::ChannelConfiguration channelConfiguration() const;
+
+      /*!
+       * Returns \c true for an Audio Data Transport Stream (ADTS), usually AAC.
+       */
+      bool isADTS() const;
+
+      /*!
+       * Returns \c true if the copyrighted bit is set.
        */
       bool isCopyrighted() const;
 
       /*!
-       * Returns true if the "original" bit is set.
+       * Returns \c true if the "original" bit is set.
        */
       bool isOriginal() const;
 
     private:
-      Properties(const Properties &);
-      Properties &operator=(const Properties &);
-
-      void read(File *file);
+      void read(File *file, ReadStyle readStyle);
 
       class PropertiesPrivate;
-      PropertiesPrivate *d;
+      TAGLIB_MSVC_SUPPRESS_WARNING_NEEDS_TO_HAVE_DLL_INTERFACE
+      std::unique_ptr<PropertiesPrivate> d;
     };
-  }
-}
+  }  // namespace MPEG
+}  // namespace TagLib
 
 #endif
