@@ -26,11 +26,11 @@
 #ifndef TAGLIB_FILE_H
 #define TAGLIB_FILE_H
 
+#include "tbytevector.h"
+#include "tiostream.h"
 #include "taglib_export.h"
 #include "taglib.h"
 #include "tag.h"
-#include "tbytevector.h"
-#include "tiostream.h"
 
 namespace TagLib {
 
@@ -66,8 +66,8 @@ namespace TagLib {
      * Specify which tags to strip either explicitly, or on save.
      */
     enum StripTags {
-      StripNone,  //<! Don't strip any tags
-      StripOthers //<! Strip all tags not explicitly referenced in method call
+      StripNone,  //!< Don't strip any tags
+      StripOthers //!< Strip all tags not explicitly referenced in method call
     };
 
     /*!
@@ -75,8 +75,8 @@ namespace TagLib {
      * types should be synchronized.
      */
     enum DuplicateTags {
-      Duplicate,     //<! Synchronize values between different tag types
-      DoNotDuplicate //<! Do not synchronize values between different tag types
+      Duplicate,     //!< Synchronize values between different tag types
+      DoNotDuplicate //!< Do not synchronize values between different tag types
     };
 
     /*!
@@ -84,6 +84,9 @@ namespace TagLib {
      */
     virtual ~File();
 
+    File(const File &) = delete;
+    File &operator=(const File &) = delete;
+
     /*!
      * Returns the file name in the local file system encoding.
      */
@@ -105,23 +108,21 @@ namespace TagLib {
      * to remove (a subset of) them.
      * For files that contain more than one tag (e.g. an MP3 with both an ID3v1 and an ID3v2
      * tag) only the most "modern" one will be exported (ID3v2 in this case).
-     * BIC: Will be made virtual in future releases.
      */
-    PropertyMap properties() const;
+    virtual PropertyMap properties() const;
 
     /*!
      * Removes unsupported properties, or a subset of them, from the file's metadata.
      * The parameter \a properties must contain only entries from
      * properties().unsupportedData().
-     * BIC: Will be mad virtual in future releases.
      */
-    void removeUnsupportedProperties(const StringList& properties);
+    virtual void removeUnsupportedProperties(const StringList& properties);
 
     /*!
      * Sets the tags of this File to those specified in \a properties. Calls the
      * according specialization method in the subclasses of File to do the translation
      * into the format-specific details.
-     * If some value(s) could not be written imported to the specific metadata format,
+     * If some value(s) could not be written to the specific metadata format,
      * the returned PropertyMap will contain those value(s). Otherwise it will be empty,
      * indicating that no problems occurred.
      * With file types that support several tag formats (for instance, MP3 files can have
@@ -129,9 +130,30 @@ namespace TagLib {
      * (ID3v2 for MP3 files). Older formats will be updated as well, if they exist, but won't
      * be taken into account for the return value of this function.
      * See the documentation of the subclass implementations for detailed descriptions.
-     * BIC: will become pure virtual in the future
      */
-    PropertyMap setProperties(const PropertyMap &properties);
+    virtual PropertyMap setProperties(const PropertyMap &properties);
+
+    /*!
+     * Get the keys of complex properties, i.e. properties which cannot be
+     * represented simply by a string.
+     * The default implementation calls Tag::complexPropertyKeys().
+     * \see Tag::complexPropertyKeys()
+     */
+    virtual StringList complexPropertyKeys() const;
+
+    /*!
+     * Get the complex properties for a given \a key.
+     * The default implementation calls Tag::complexProperties().
+     * \see Tag::complexProperties()
+     */
+    virtual List<VariantMap> complexProperties(const String &key) const;
+
+    /*!
+     * Set all complex properties for \a key using the variant maps \a value.
+     * The default implementation calls Tag::setComplexProperties().
+     * \see Tag::setComplexProperties()
+     */
+    virtual bool setComplexProperties(const String &key, const List<VariantMap> &value);
 
     /*!
      * Returns a pointer to this file's audio properties.  This should be
@@ -142,7 +164,7 @@ namespace TagLib {
 
     /*!
      * Save the file and its associated tags.  This should be reimplemented in
-     * the concrete subclasses.  Returns true if the save succeeds.
+     * the concrete subclasses.  Returns \c true if the save succeeds.
      *
      * \warning On UNIX multiple processes are able to write to the same file at
      * the same time.  This can result in serious file corruption.  If you are
@@ -155,11 +177,11 @@ namespace TagLib {
     /*!
      * Reads a block of size \a length at the current get pointer.
      */
-    ByteVector readBlock(unsigned long length);
+    ByteVector readBlock(size_t length);
 
     /*!
      * Attempts to write the block \a data at the current get pointer.  If the
-     * file is currently only opened read only -- i.e. readOnly() returns true --
+     * file is currently only opened read only -- i.e. readOnly() returns \c true --
      * this attempts to reopen the file in read/write mode.
      *
      * \note This should be used instead of using the streaming output operator
@@ -180,8 +202,8 @@ namespace TagLib {
      * \note This has the practical limitation that \a pattern can not be longer
      * than the buffer size used by readBlock().  Currently this is 1024 bytes.
      */
-    long find(const ByteVector &pattern,
-              long fromOffset = 0,
+    offset_t find(const ByteVector &pattern,
+              offset_t fromOffset = 0,
               const ByteVector &before = ByteVector());
 
     /*!
@@ -196,8 +218,8 @@ namespace TagLib {
      * \note This has the practical limitation that \a pattern can not be longer
      * than the buffer size used by readBlock().  Currently this is 1024 bytes.
      */
-    long rfind(const ByteVector &pattern,
-               long fromOffset = 0,
+    offset_t rfind(const ByteVector &pattern,
+               offset_t fromOffset = 0,
                const ByteVector &before = ByteVector());
 
     /*!
@@ -207,7 +229,7 @@ namespace TagLib {
      * \note This method is slow since it requires rewriting all of the file
      * after the insertion point.
      */
-    void insert(const ByteVector &data, unsigned long start = 0, unsigned long replace = 0);
+    void insert(const ByteVector &data, offset_t start = 0, size_t replace = 0);
 
     /*!
      * Removes a block of the file starting a \a start and continuing for
@@ -216,10 +238,10 @@ namespace TagLib {
      * \note This method is slow since it involves rewriting all of the file
      * after the removed portion.
      */
-    void removeBlock(unsigned long start = 0, unsigned long length = 0);
+    void removeBlock(offset_t start = 0, size_t length = 0);
 
     /*!
-     * Returns true if the file is read only (or if the file can not be opened).
+     * Returns \c true if the file is read only (or if the file can not be opened).
      */
     bool readOnly() const;
 
@@ -230,7 +252,7 @@ namespace TagLib {
     bool isOpen() const;
 
     /*!
-     * Returns true if the file is open and readable.
+     * Returns \c true if the file is open and readable.
      */
     bool isValid() const;
 
@@ -240,7 +262,7 @@ namespace TagLib {
      *
      * \see Position
      */
-    void seek(long offset, Position p = Beginning);
+    void seek(offset_t offset, Position p = Beginning);
 
     /*!
      * Reset the end-of-file and error flags on the file.
@@ -250,37 +272,22 @@ namespace TagLib {
     /*!
      * Returns the current offset within the file.
      */
-    long tell() const;
+    offset_t tell() const;
 
     /*!
      * Returns the length of the file.
      */
-    long length();
-
-    /*!
-     * Returns true if \a file can be opened for reading.  If the file does not
-     * exist, this will return false.
-     *
-     * \deprecated Use system functions, e.g. access() (_access_s() on Windows).
-     */
-    TAGLIB_DEPRECATED static bool isReadable(const char *file);
-
-    /*!
-     * Returns true if \a file can be opened for writing.
-     *
-     * \deprecated Use system functions, e.g. access() (_access_s() on Windows).
-     */
-    TAGLIB_DEPRECATED static bool isWritable(const char *name);
+    offset_t length();
 
   protected:
     /*!
-     * Construct a File object and opens the \a file.  \a file should be a
-     * be a C-string in the local file system encoding.
+     * Construct a File object and open the \a fileName.  \a fileName should be a
+     * C-string in the local file system encoding.
      *
      * \note Constructor is protected since this class should only be
      * instantiated through subclasses.
      */
-    File(FileName file);
+    File(FileName fileName);
 
     /*!
      * Construct a File object and use the \a stream instance.
@@ -303,7 +310,7 @@ namespace TagLib {
     /*!
      * Truncates the file to a \a length.
      */
-    void truncate(long length);
+    void truncate(offset_t length);
 
     /*!
      * Returns the buffer size that is used for internal buffering.
@@ -311,11 +318,9 @@ namespace TagLib {
     static unsigned int bufferSize();
 
   private:
-    File(const File &);
-    File &operator=(const File &);
-
     class FilePrivate;
-    FilePrivate *d;
+    TAGLIB_MSVC_SUPPRESS_WARNING_NEEDS_TO_HAVE_DLL_INTERFACE
+    std::unique_ptr<FilePrivate> d;
   };
 
 }  // namespace TagLib
